(*
 * <COPYRIGHT>
 * Copyright 2005 Gerd Stolpmann
 *
 * <GPL>
 * This file is part of UMLMON.
 *
 * UMLMON is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * UMLMON is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with UMLMON; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * </>
 *)

(* $Id: linux.mli 250 2007-06-28 14:42:41Z gerd $ *)

module Privilege : sig
  external initgroups : string -> int -> unit = "unix_initgroups"
    (* See initgroups(3) *)

  external getgrouplist : string -> int -> int array = "unix_getgrouplist"
    (* See getgrouplist(3). Note that this is a GNU extension. *)

  external setreuid : int -> int -> unit = "umlmon_setreuid"
  external setregid : int -> int -> unit = "umlmon_setregid"
end


module Cap : sig
  (* Access to capabilities *)
  (* external drop_privileges : unit -> unit = "unix_drop_privileges" *)
end


module Sysconf : sig
  (* Access to system configuration/limits *)
  external pagesize : unit -> int = "misc_sysconf_pagesize"
end


module Pty : sig
  (* Helpers for pseudo terminals *)
  external grantpt : Unix.file_descr -> unit = "unix_grantpt"
  external unlockpt : Unix.file_descr -> unit = "unix_unlockpt"
  external ptsname : Unix.file_descr -> string = "unix_ptsname"
end


module File : sig
  (* File-related functions *)
  external realpath : string -> string = "unix_realpath"
    (* Returns the absolute path without ".", "..", "//", and symlinks, i.e.
     * the physical path.
     *)

  type node_type = 
    | S_IFREG 
    | S_IFCHR of int  (* major + minor *)
    | S_IFBLK of int  (* major + minor *)
    | S_IFIFO
    | S_IFSOCK

  external mknod : string -> int -> node_type -> unit = "unix_mknod"
    (* Creates the node with the given permissions and the given type *)

  val copy : string -> string -> unit
    (* Copies a single file *)
end


module Process : sig
  (* Process-related calls *)

  external set_sched_other : int option -> unit = "ml_set_sched_other"
    (* Sets the SCHED_OTHER scheduler, and optionally sets the "nice"-type
     * priority.
     *)

  external set_sched_rr : int -> unit = "ml_set_sched_rr"
    (* Sets the SCHED_RR scheduler. The int is the static priority.
     *)
(*
  external set_affinity : int array -> unit = "ml_set_affinity"
    (* The array enumerates the affine CPUs *)
 *)
end


module Tun : sig
  (* TUN/TAP network interfaces *)
  external create_tap : Unix.file_descr -> string -> int -> string
    = "tun_create_tap"
    (** [tun_create_tap tun name owner] creates the TAP network device for the
     * open [tun] device with passed [name], owner by [owner].
     *
     * [tun] should be /dev/net/tun, r/w-opened.
     * The [name] should have the form [tapN] where [N] is a number, or [""].
     *
     * The returned string is the actual name of the TAP device.
     *)
 
  external delete_tap : Unix.file_descr -> string -> unit
    = "tun_delete_tap"
    (** [tun_delete_tap tun name] deletes the named TAP device for the open
     * [tun] device.
     *)
end
