(* $Id: rndsrv.ml 252 2007-06-28 15:42:55Z gerd $ *)

(* Open /dev/urandom on the host, and make a slowed-down version of this
   stream available over Unix domain socket 
 *)

let rnddev = "/dev/urandom"
let socket = "/tmp/rndsrv.sock"


let fork_and_serve so =
  match Unix.fork() with
    | 0 ->
	( try
	    let fd = Unix.openfile rnddev [Unix.O_RDONLY] 0 in
	    let buf = String.create 8192 in
	    let eof = ref false in
	    while not !eof do
	      let n = Unix.read fd buf 0 (String.length buf) in
	      if n > 0 then (
		let m = ref 0 in
		while !m < n do
		  let k = Unix.single_write so buf !m (n - !m) in
		  m := !m + k
		done;
		ignore(Unix.select [] [] [] 1.0);
	      )
	      else eof := true
	    done
	  with
	    | Unix.Unix_error((Unix.EPIPE | Unix.ECONNRESET),_,_) ->
		()
	    | error ->
		 prerr_endline ("Exception: " ^ Printexc.to_string error)
	);
	exit 0
    | _ ->
	()


let main() =
  Sys.set_signal Sys.sigpipe Sys.Signal_ignore;
  Sys.set_signal Sys.sigchld Sys.Signal_ignore;
  ( try Sys.remove socket with _ -> () );
  let so = Unix.socket Unix.PF_UNIX Unix.SOCK_STREAM 0 in
  Unix.bind so (Unix.ADDR_UNIX socket);
  Unix.chmod socket 0o777;
  Unix.listen so 10;
  while true do
    try
      let so',_ = Unix.accept so in
      fork_and_serve so';
      Unix.close so'
    with
      | _ -> ()
  done
;;


main()
