(*
 * <COPYRIGHT>
 * Copyright 2005 Gerd Stolpmann
 *
 * <GPL>
 * This file is part of UMLMON.
 *
 * UMLMON is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * UMLMON is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with UMLMON; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * </>
 *)

(* $Id: umlmon_shared.mli 209 2005-10-23 13:46:46Z gerd $ *)

(* Management of the "shared" directory *)

open Umlmon_types

val setup_shared : umlglobalspec -> unit
  (* Ensures that the "shared" directory exists and contains the files
   * needed at minimum:
   * - shared/disks: directory
   * - shared/disks/<devnode>: block devices
   *   If configured, it is checked whether the device identity is still
   *   the same, and optionally, the device identities are updated.
   * - shared/locks: file to record locks
   * - shared/kernels: directory
   *)

type lock = [ `Read | `Write ]

val lock_disks : umlglobalspec -> umlspec -> (lock * jailfile) list -> lock_id
  (* Locks the disks. The jailfiles must have the form "/shared/disks/<node>".
   * Raises [Lock_failure] if the locks cannot be granted.
   *)

val unlock_disks : umlglobalspec -> umlspec -> lock_id -> unit
  (* Unlocks the disks. Nothing happens if we do not have the lock. *)

val unlock_all_disks : umlglobalspec -> umlspec -> unit
  (* Unlocks all disks locked by this VM. *)

val get_locks : umlglobalspec -> umlspec -> (lock * jailfile * bool) list
  (* Returns the list of locks. The bool tells us whether the 
   * lock is held by the passed VM.
   *)

val reserve_hwaddr : umlglobalspec -> umlspec -> string option -> hwaddr
  (* Returns a MAC address from the pool. The passed string is the optional
   * IP address (as dotted quad). If given, it is tried to use the same
   * address as the last time. The MAC address is returned as string of
   * six bytes.
   *)

val release_all_hwaddr : umlglobalspec -> umlspec -> unit
  (* Releases all MAC addresses reserved by this VM. *)

val get_kernels : umlglobalspec -> jailfile list
  (* Returns the possible kernels (in shared/kernels). The returned names
   * have the form "/shared/kernels/<name>".
   *
   * Kernels must be world-executable.
   *)
